@@ -88,7 +88,7 @@ class DependenciesNotebookNameHandler(APIHandler):
 
         home = Path.home()
         store_path: Path = home.joinpath(".local/share/thoth/kernels")
-        _LOGGER.info("Path used to store notebook handled is: %r", store_path.as_posix())
+        _LOGGER.debug("Path used to store notebook handled is: %r", store_path.as_posix())
 
         file_name = "thoth_notebook_tracker.json"
 
@@ -100,6 +100,8 @@ class DependenciesNotebookNameHandler(APIHandler):
 
         data = {"notebook_name": notebook_name}
 
+        store_path.mkdir(parents=True, exist_ok=True)
+
         with open(file_path, "w") as outfile:
             json.dump(data, outfile)
 
