@@ -33,6 +33,9 @@ from virtualenv import cli_run
 from pathlib import Path
 
 from rich.text import Text
+from rich import box
+from rich.console import Console
+from rich.table import Table
 
 from thoth.python import Project, Pipfile, PipfileLock
 from thoth.python import PipfileMeta
@@ -53,6 +56,50 @@ _EMOJI = {
 }
 
 
+def _print_report(report: dict, title: typing.Optional[str] = None):
+    """Print reasoning to user."""
+    console = Console()
+    table = Table(
+        show_header=True,
+        header_style="bold green",
+        title=title,
+        box=box.MINIMAL_DOUBLE_HEAD,
+    )
+
+    header = set()  # type: typing.Set[str]
+    to_remove = set()  # type: typing.Set[str]
+    for item in report:
+        header = header.union(set(item.keys()))
+        to_remove = to_remove.union(set(i for i, v in item.items() if isinstance(v, dict)))
+
+    # Remove fields that can be an array - these are addition details that are supressed from the table output.
+    header = header - to_remove
+
+    header_list = sorted(header)
+    for item in header_list:
+        table.add_column(item.replace("_", " ").capitalize(), style="cyan", overflow="fold")
+
+    for item in report:
+        row = []
+        for column in header_list:
+            entry = item.get(column, "-")
+
+            if not bool(int(os.getenv("JUPYTERLAB_REQUIREMENTS_NO_EMOJI", 0))) and isinstance(entry, str):
+                entry = _EMOJI.get(entry, entry)
+
+            if isinstance(entry, list):
+                entry = ", ".join(entry)
+
+            if isinstance(entry, str) and entry.startswith(("https://", "http://")):
+                entry = f"[link {entry}]{entry}"
+
+            row.append(entry)
+
+        table.add_row(*row)
+
+    console.print(table, justify="center")
+
+
 def get_notebook_content(notebook_path: str, py_format: bool = False):
     """Get JSON of the notebook content."""
     actual_path = Path(notebook_path)
@@ -82,7 +129,7 @@ def get_notebook_content(notebook_path: str, py_format: bool = False):
         return notebook_content_py
 
 
-def check_metadata_content(notebook_metadata: dict, is_cli: bool = True) -> list:
+def horus_check_metadata_content(notebook_metadata: dict, is_cli: bool = True) -> list:
     """Check the metadata of notebook for dependencies."""
     result = []
 
@@ -179,7 +226,7 @@ def check_metadata_content(notebook_metadata: dict, is_cli: bool = True) -> list
         else:
             result.append(
                 {
-                    "message": f"{mandatory_key} key is present in notebook metadata",
+                    "message": f"{mandatory_key} key is present in notebook metadata.",
                     "type": "INFO",
                 }
             )
@@ -242,7 +289,7 @@ def check_metadata_content(notebook_metadata: dict, is_cli: bool = True) -> list
         if kernel_packages:
             for package in notebook_packages:
                 if str(package.name) in kernel_packages:
-                    if str(package.version) in kernel_packages[str(package.name)]:
+                    if str(package.version).strip("==") in kernel_packages[str(package.name)]:
                         check += 1
                 else:
                     break
@@ -250,8 +297,9 @@ def check_metadata_content(notebook_metadata: dict, is_cli: bool = True) -> list
         if check == len([p for p in notebook_packages]):
             result.append(
                 {
-                    "message": f"kernel {kernel_name} selected has all dependencies installed.",
-                    "type": "INFO",
+                    "message": f"kernel {kernel_name} matches packages(name,version,PyPI index) "
+                    "from requirements lock in your notebook metadata.",
+                    "type": "WARNING",
                 }
             )
         else:
@@ -338,6 +386,17 @@ def get_packages(kernel_name: str, kernels_path: Path = Path.home().joinpath(".l
                 package_version = [el for el in processed_package.split(" ") if el != ""]
                 packages[package_version[0]] = package_version[1]
 
+    # default kernel for Jupyter
+    if kernel_name == "python3":
+        process_output = subprocess.run("pip list", shell=True, capture_output=True)
+
+        processed_list = process_output.stdout.decode("utf-8").split("\n")[2:]
+
+        for processed_package in processed_list:
+            if processed_package:
+                package_version = [el for el in processed_package.split(" ") if el != ""]
+                packages[package_version[0]] = package_version[1]
+
     return packages
 
 
@@ -378,7 +437,27 @@ def create_kernel(kernel_name: str, kernels_path: Path = Path.home().joinpath(".
         _LOGGER.error(f"Could not enter environment {e}")
 
 
-def delete_kernel(kernel_name: str, kernels_path: Path = Path.home().joinpath(".local/share/thoth/kernels")):
+def horus_list_kernels(kernels_path: Path = Path.home().joinpath(".local/share/thoth/kernels")) -> typing.List[str]:
+    """List kernels from host."""
+    # List jupyter kernel
+    try:
+        command_output = subprocess.run(
+            "jupyter kernelspec list --json",
+            shell=True,
+            capture_output=True,
+        )
+        _LOGGER.debug(command_output.returncode)
+
+    except Exception as e:
+        _LOGGER.error(f"Kernel list could not be obtained: {e}")
+
+    # Delete folder from host
+    kernels = [k for k in json.loads(command_output.stdout.decode("utf-8"))["kernelspecs"]]
+
+    return kernels
+
+
+def horus_delete_kernel(kernel_name: str, kernels_path: Path = Path.home().joinpath(".local/share/thoth/kernels")):
     """Delete kernel from host."""
     # Delete jupyter kernel
     try:
@@ -404,7 +483,7 @@ def delete_kernel(kernel_name: str, kernels_path: Path = Path.home().joinpath(".
     return command_output
 
 
-def gather_libraries(notebook_path: str):
+def gather_libraries(notebook_path: str) -> typing.List[str]:
     """Gather libraries with invectio."""
     library_gathered = []
     try:
@@ -463,6 +542,7 @@ def lock_dependencies_with_thoth(
     config: str,
     timeout: int,
     force: bool,
+    debug: bool,
     notebook_content: str,
     kernels_path: Path = Path.home().joinpath(".local/share/thoth/kernels"),
 ) -> typing.Tuple[int, dict]:
@@ -482,7 +562,7 @@ def lock_dependencies_with_thoth(
     _LOGGER.info("Current path: %r ", env_path.as_posix())
     _LOGGER.info(f"Input Pipfile: \n{pipfile_string}")
 
-    advise = {"requirements": {}, "requirement_lock": {}, "error": False, "error_msg": ""}
+    advise = {"requirements": {}, "requirement_lock": {}, "error": False, "error_msg": "", "stack_info": {}}
     returncode = 0
 
     temp = tempfile.NamedTemporaryFile(prefix="jl_thoth_", mode="w+t")
@@ -506,6 +586,7 @@ def lock_dependencies_with_thoth(
             no_static_analysis=False,
             timeout=timeout,
             src_path=temp.name,
+            debug=debug,
         )
 
         _LOGGER.info(f"Response: {response}")
@@ -532,7 +613,13 @@ def lock_dependencies_with_thoth(
                 pipfile = result["report"]["products"][0]["project"]["requirements"]
                 pipfile_lock = result["report"]["products"][0]["project"]["requirements_locked"]
 
-                advise = {"requirements": pipfile, "requirement_lock": pipfile_lock, "error": False}
+                advise["requirements"] = pipfile
+                advise["requirement_lock"] = pipfile_lock
+                advise["error"] = False
+                advise["justification"] = result["report"]["products"][0]["justification"]
+
+            if result["report"] and result["report"]["stack_info"]:
+                advise["stack_info"] = result["report"]["stack_info"]
 
     except Exception as api_error:
         _LOGGER.debug(f"error locking dependencies using Thoth: {api_error}")
@@ -561,7 +648,7 @@ def lock_dependencies_with_thoth(
                 _LOGGER.info("Writing to Pipfile/Pipfile.lock in %r", env_path.as_posix())
                 project.to_files(pipfile_path=pipfile_path, pipfile_lock_path=pipfile_lock_path)
         except Exception as e:
-            _LOGGER.debug("Requirements files have not been stored successfully %r", e)
+            _LOGGER.debug("Requirements files cannot be stored due to: %r", e)
 
     os.chdir(initial_path)
 
@@ -707,7 +794,7 @@ def horus_requirements_command(
     dev: bool,
     add: typing.Optional[typing.List[str]] = None,
     remove: typing.Optional[typing.List[str]] = None,
-    save: bool = True,
+    save_in_notebook: bool = True,
 ):
     """Horus requirements command."""
     notebook = get_notebook_content(notebook_path=path)
@@ -757,7 +844,7 @@ def horus_requirements_command(
                 )
                 sys.exit(1)
 
-    if save:
+    if save_in_notebook:
         notebook_metadata["requirements"] = json.dumps(pipfile_.to_dict())
 
         notebook["metadata"] = notebook_metadata
@@ -766,7 +853,7 @@ def horus_requirements_command(
     return pipfile_
 
 
-def create_pipfile_from_packages(packages: list, python_version: str):
+def create_pipfile_from_packages(packages: list, python_version: str) -> Pipfile:
     """Create Pipfile from list of packages."""
     source = Source(url="https://pypi.org/simple", name="pypi", verify_ssl=True)
 
@@ -875,6 +962,7 @@ def horus_lock_command(
     resolution_engine: str,
     timeout: int,
     force: bool,
+    debug: bool,
     recommendation_type: str,
     kernel_name: typing.Optional[str] = None,
     os_name: typing.Optional[str] = None,
@@ -891,8 +979,11 @@ def horus_lock_command(
     notebook = get_notebook_content(notebook_path=path)
     notebook_metadata = notebook.get("metadata")
 
-    kernelspec = notebook_metadata.get("kernelspec")
-    notebook_kernel = kernelspec.get("name")
+    if notebook_metadata.get("kernelspec"):
+        kernelspec = notebook_metadata.get("kernelspec")
+        notebook_kernel = kernelspec.get("name")
+    else:
+        kernel_name = "python3"
 
     if not kernel_name:
         kernel = notebook_kernel
@@ -969,6 +1060,7 @@ def horus_lock_command(
             config=json.dumps(thoth_config.content),
             timeout=timeout,
             force=force,
+            debug=debug,
             notebook_content=notebook_content_py,
         )
 
@@ -998,19 +1090,7 @@ def horus_lock_command(
 
         _LOGGER.info("Path used to store dependencies is: %r", complete_path.as_posix())
 
-        requirements_format = "pipenv"
-
-        if resolution_engine == "thoth":
-            project = Project.from_dict(requirements, requirements_lock)
-        else:
-            project = Project.from_dict(pipfile_.to_dict(), requirements_lock)
-
-        pipfile_path = complete_path.joinpath("Pipfile")
-        pipfile_lock_path = complete_path.joinpath("Pipfile.lock")
-
-        if requirements_format == "pipenv":
-            _LOGGER.debug("Writing to Pipfile/Pipfile.lock in %r", complete_path)
-            project.to_files(pipfile_path=pipfile_path, pipfile_lock_path=pipfile_lock_path)
+        # requirements and requirements locked are already stored at this point
 
         if resolution_engine == "thoth":
             # thoth
@@ -1022,8 +1102,8 @@ def horus_lock_command(
 
     if save_in_notebook and not error:
         notebook_metadata["dependency_resolution_engine"] = resolution_engine
-        notebook_metadata["requirements"] = requirements
-        notebook_metadata["requirements_lock"] = requirements_lock
+        notebook_metadata["requirements"] = json.dumps(requirements)
+        notebook_metadata["requirements_lock"] = json.dumps(requirements_lock)
 
         # Assign kernel name to kernelspec.
         kernelspec["name"] = kernel
@@ -1041,6 +1121,7 @@ def horus_set_kernel_command(
     save_in_notebook: bool = True,
     resolution_engine: typing.Optional[str] = None,
     is_magic_command: bool = False,
+    force: bool = False,
 ):
     """Create kernel using dependencies in notebook metadata."""
     results = {}
@@ -1085,9 +1166,9 @@ def horus_set_kernel_command(
 
     complete_path: Path = store_path.joinpath(kernel)
 
-    if not is_magic_command:
+    if not is_magic_command or force:
         if complete_path.exists():
-            delete_kernel(kernel_name=kernel)
+            horus_delete_kernel(kernel_name=kernel)
 
         complete_path.mkdir(parents=True, exist_ok=True)
 
@@ -1123,7 +1204,7 @@ def horus_set_kernel_command(
         is_magic_command=is_magic_command,
     )
 
-    # 3. Install packages using micropipenv
+    # 3. Assign virtualenv to jupyter kernel
     create_kernel(kernel_name=kernel)
 
     if save_in_notebook:
